body {
background-color:#FFF;
font-family:"liberation sans", "Bitstream Vera Sans", "Luxi Sans", helvetica, verdana, arial, sans-serif;
font-size:9pt;
line-height:150%;
color:#555;
position:relative;
float:left;
margin:0;
padding:1em;
width: 100%;
}


.static_tocnavwrap ul {
display:block;
clear:both;
}

.static_tocnavwrap ul li {
list-style:none;
display:block;
width: 5em;
}

.static_tocnavwrap ul li a {
text-decoration:none;
display:block;
margin:1px;
padding:2px 5px;
}

.static_tocnavwrap ul li a:link,
.static_tocnavwrap ul li a:visited {
background-color:#eee;
}

.static_tocnavwrap ul li a:hover {
background-color:#000;
}

.static_tocnavwrap span.product {
font-weight:700;
}

.static_tocnavwrap .product {
display:block;
clear:both !important;
background-image:none !important;
}

.static_tocnavwrap .book {
}

.static_tocnavwrap div.book {
margin-bottom:0.5em;
}

.static_tocnavwrap h1 {
text-align:center;
font-size:3em;
color:#000;
}

.static_tocnavwrap h2 {
text-align:center;
font-size:2em;
color:#000;
}

.static_tocnavwrap .version {
background-color:#eee;
}

