html {
    overflow-y: scroll;
}
body {
    border: 40px solid #C3C3C3;
    margin: 0;
    padding: 0 10px;
	font-family: "liberation sans", "Myriad ", "Bitstream Vera Sans", "Lucida Grande", "Luxi Sans", "Trebuchet MS", helvetica, verdana, arial, sans-serif;
}

body.menu_embeded {
    margin: 0 auto;
    max-width: 100%;
    padding-left: 0px;
    padding-right: 0px;
}

body.toc_embeded {
	/*for web hosting system only*/
	padding: 0em 100px 0px 300px;
	margin: 0 auto;
}

body[dir="rtl"].toc_embeded {
	/*for web hosting system only*/
	padding: 0em 100px 0px 300px;
}

body.notoc {
	margin-left:3em;
	padding-left: 0px;
}

body[dir="rtl"].notoc {
	margin-right:3em;
	padding-right: 0px;
}

a:focus {
    outline: medium none;
}
body > div.book, #main > div.book,
body > div.article, #main > div.article,
body > div.set, #main > div.set,
body > div.preface, #main > div.preface,
body > div.chapter, #main > div.chapter,
body > div.section, #main > div.section,
body > div.appendix, #main > div.appendix,
body > div.part, #main > div.part,
body > div.index {
    margin-left: auto;
    margin-right: auto;
    max-width: 45em;
    margin-top: 40px;
    margin-bottom:4em;
/*
-webkit-box-shadow: 0 0 0 1px #AAAAAA, 0 0 10px 0 #777777;
   -moz-box-shadow: 0 0 0 1px #AAAAAA, 0 0 10px 0 #777777;
        box-shadow: 0 0 0 1px #AAAAAA, 0 0 10px 0 #777777;
*/
-webkit-box-shadow: 0 .5em 3em #aaa;
   -moz-box-shadow: 0 .5em 3em #aaa;
        box-shadow: 0 .5em 3em #aaa;

    background-color: white;
    padding-left: 30px;
    padding-right: 40px;
    padding-bottom: 20px;
    padding-top: 1px;
}

body[dir="rtl"] > div.book,
body[dir="rtl"] > div.article,
body[dir="rtl"] > div.set,
body[dir="rtl"] > div.preface,
body[dir="rtl"] > div.chapter,
body[dir="rtl"] > div.section,
body[dir="rtl"] > div.appendix,
body[dir="rtl"] > div.part {
    padding-right: 30px;
    padding-left: 40px;
}

body.toc_embeded > div.book,
body.toc_embeded > div.article,
body.toc_embeded > div.set,
body.toc_embeded > div.preface,
body.toc_embeded > div.chapter,
body.toc_embeded > div.section,
body.toc_embeded > div.appendix,
body.toc_embeded > div.part {
	width: 50em;
	padding-top: 1px;
}

.lang_top_box {
/*    margin-top: 4px;*/
}
.langauge_banner {
    border: 1px solid #C3C3C3;
    height: 100px;
    margin-bottom: 10px;
}
.popular_books {
    background-color: #787878;
    height: 50px;
}
.lang_bottom_box {
    background-color: white;
    height: 100%;
    margin-left: 240px !important;
    padding-left: 30px;
    padding-top: 20px;
    position: relative;
    top: 0;
    padding-right: 30px;
}

body[dir="rtl"] .lang_bottom_box {
    margin-left: 0px !important;
    margin-right: 240px !important;
}

.lang_bottom_box:after, .product_bottom_box:after {
    clear: both !important;
    content: "";
    display: block;
    height: 20px;
}
.side_products {
    color: #555555;
    background-color: #F5F5F5;
    float: left;
    padding-top: 20px;
    position: relative;
    width: 240px;
}

body[dir="rtl"] .side_products {
    float: right;
}

.side_products:after {
    clear: both;
    content: " ";
}
.side_products div.link {
    font-size: 12px;
/*    height: 24px; */
    line-height: 1em;
/*    margin-bottom: 10px; */
}
.side_products div.link a {
    color: #333333;
    display: block;
    text-decoration: none;
    padding-left: 25px;
}

body[dir="rtl"] .side_products div.link a {
    padding-left: 0px;
    padding-right: 25px;
}

.side_products div.link a.active {
    background-color: #333333;
    color: #EEEEEE;
    margin:0em;
    position: relative;
}
.side_products div.link a.active { position:relative;
    color: #EEEEEE;
}
.side_products div.link a.active:after {
    margin: 0em;
    left: 100%;
    top: 50%;
    margin-top: -1em;
    border-color: transparent transparent transparent #333;
    border-style: solid;
    border-width: 1em 0em 1em 1em !important;
    content: "";
    display: inline-block;
    position: absolute;
    width: 0;
    z-index: 9;
}

body[dir="rtl"] .side_products div.link a.active:after {
    border-color: transparent #333 transparent transparent;
    border-width: 1em 1em 1em 0em !important;
}

.main_content {
/*    background-color: #F5F5F5; */
    clear: both !important;
    position: relative;
}
.main_content:after {
    clear: both;
    content: "";
    display: block;
    height: 2px;
    width: 100%;
}
.bottom_products {
    display: block;
    font-size: 1.4em;
    margin-top: 20px;
}

.bottom_products .title {
    text-align: center;
}

.bottom_products .title span {
    background-color: #FFFFFF !important;
    color: #999999 !important;
}
.bottom_versions {
    display: block;
    padding-bottom: 10px;
    padding-top: 10px;
}
.bottom_versions:last-child .bottom_books:after {
    clear: both !important;
    content: "";
    display: block;
    width: 100%;
}
.menu .bottom_versions .title span {
    background-color: #EEEEEE;
    display: inline-block;
    left: -20px;
    line-height: 30px;
    margin: 0;
    padding-top: 4px;
    position: relative;
    width: 100%;
}

body[dir="rtl"] .menu .bottom_versions .title span {
    right: -20px;
}

.menu span a, #menu span a {
    color: white;
}

.product_bottom_box {
}

.product_top_box,
.version_top_box {
    min-height: 100px;
    color: #555555;
    font-size: 16px;
    padding-left: 20px;
    padding-right: 20px;
    line-height: 1.25em;
    padding-bottom: 10px;
}

.version_top_box .para,
.product_top_box .para,
.product_top_box .formalpara {
    margin-bottom: 10px;
}


.product_top_box>:first-child,
.version_top_box>:first-child {
    font-size: 18px;
    text-align: center;
}


.product_top_box h5 {
    font-size: 18px;
}


.version_bottom_box {
}
.book_top_box {
    height: 100px;
    margin-bottom: 10px;
}
.book_bottom_box {
    min-height: 200px;
}
.book_versions {
    float: left;
    margin: 0;
    width: 45%;
}

body[dir="rtl"] .book_versions {
    float: right;
}

.book_version {
    background-color: #A3A3A3;
    float: left;
    margin: 5px;
    padding: 5px;
    text-align: center;
    width: 50%;
}

body[dir="rtl"] .book_version {
    float: right;
}
 
.book_formats {
    background-color: #111111;
    float: right;
    margin: 0;
    padding: 20px;
    width: 100px;
}

body[dir="rtl"] .book_formats {
    float: left;
}

.book_formats div {
    margin-bottom: 20px;
}
.book_formats a {
    color: white;
    margin-bottom: 20px;
}
#menu {
    background-color: #666666;
    color: #EEEEEE;
    font-size: 13px;
    height: 30px !important;
    left: 0;
    line-height: 30px;
    position: fixed;
    top: 0;
    width: 100%;
}

body[dir="rtl"] #menu {
    right: 0;
}

#menu ul {
    list-style: none outside none !important;
    margin: 0px;
    padding: 0px;
    min-width: 100%;
}

#menu ul li {
    margin-left: 5px;
    margin-right: 5px;
}

#menu a {
    border: medium none !important;
    color: #444444;
    white-space: pre;
    display: block;
    text-decoration: none;
}

#menu ul a,
#lang_menu:hover a {
    color: white;
}

.breadcrumb {
    padding-left: 10px;
    padding-right: 10px;
}
#langauge_menu {
    background-color: red;
    left: 0;
    position: absolute;
    width: 50px;
    z-index: 9;
}

body[dir="rtl"] #langauge_menu {
    right: 0;
}

#product_menu {
    display: inline-block;
    z-index: 19;
}
#product_menu_list {
    position: absolute;
    top: 10px;
    z-index: 99;

}
#version_menu {
    display: inline-block;
    z-index: 18;
}
#version_menu_list {
    position: absolute;
    top: 10px;
/*     width: 350px;*/
    z-index: 98;
}
#book_menu {
    display: inline-block;
    z-index: 17;
}
#book_menu_list {
    position: absolute;
    top: 10px;
    z-index: 97;
}
#book_lang_menu, #lang_menu_2, #book_format_menu {
    background-color: transparent;
    border-left: 1px solid rgba(0, 0, 0, 0.2);
    border-right: 1px solid rgba(0, 0, 0, 0.2);
    float: right;
    height: 100%;
    padding-right: 10px;
    position: relative;
    right: 240px;
    z-index: 19;
}

body[dir="rtl"] #book_lang_menu,
body[dir="rtl"] #lang_menu_2,
body[dir="rtl"] #book_format_menu {
    float: left;
    padding-left: 10px;
    padding-right: 0px;
    right: auto;
    left: 240px;
}

#lang_menu_2:before,
#book_format_menu:before,
#book_lang_menu:before {
    border-color: #666666 transparent transparent;
    border-style: solid;
    border-width: 6px 6px 0;
    content: "";
    display: block;
    height: 0;
    position: absolute;
    right: 6px;
    top: 11px;
    width: 0;
}

body[dir="rtl"] #lang_menu_2:before,
body[dir="rtl"] #book_format_menu:before,
body[dir="rtl"] #book_lang_menu:before {
    right: auto;
    left: 6px;
}

#lang_menu_2 div,
#book_format_menu div,
#book_lang_menu div {
    color: #555555;
    margin: 0px;
    margin-bottom: -20px;
    padding: 0px;
    padding-left:10px;
    padding-right:20px;
}

body[dir="rtl"] #lang_menu_2 div,
body[dir="rtl"] #book_format_menu div,
body[dir="rtl"] #book_lang_menu div {
    padding-right:10px;
    padding-left:20px;
}

#book_lang_menu_list, #lang_menu_list, #book_format_menu_list {
    background-color: rgba(110, 110, 110, 0.95) !important;
    line-height: 32px;
    margin-top: 0 !important;
    position: absolute;
    text-decoration: none;
    top: 32px;
    z-index: 99;
}
#book_lang_menu_list a,
#lang_menu_list a,
#book_format_menu_list a {
    text-decoration: none;
}
.untranslated {
    font-style: italic;
}
.untranslated span {
    opacity: 0.5;
    margin-left: 10px;
    padding-right: 5px !important;
}

body[dir="rtl"] .untranslated span {
    margin-left: 0px;
    margin-right: 10px;
    padding-left: 5px !important;
    padding-right: 0px !important;
}

.hidden {
    display: none;
}
.version_tabs {
    height: 42px;
    width: 99%;
}
.version_tabs .link {
    color: #555555;
    display: inline-block;
    float: right;
    font-size: 16px;
    font-weight: bold;
    position: relative;
-webkit-box-shadow: 0 0 2px white;
   -moz-box-shadow: 0 0 2px white;
        box-shadow: 0 0 2px white;
}

body[dir="rtl"] .version_tabs .link {
    float: left;
}

.version_tabs .link a {
    background-color: #666666;
    display: block;
    padding: 5px 10px;
}
.version_tabs .link a.active:before {
    border-color: transparent transparent #FFFFFF;
    border-style: solid;
    border-width: 6px 8px;
    bottom: 0px;
    content: "";
    display: inline-block;
    left: 50%;
    margin-left: -7px;
    position: absolute;
    width: 0;
    z-index: 9;
}

body[dir="rtl"] .version_tabs .link a.active:before {
    right: 50%;
    margin-right: -10px;
    margin-left: 0px;
}

.version_tabs .link:last-child:after {
    clear: both;
    content: "";
}
.version_tabs .link a.active {
    background-color: #333333;
    color: #EEEEEE;
    position: relative;
}
.bottom_products span, .bottom_products span a, .bottom_products {
    margin: 0;
    padding: 0;
}
body {
    background-color: #EEEEEE;
    border: medium none;
    font-size: 1em;
    padding-top: 32px;
}

body > #wrapper {
    background-color: #FFFFFF;
    margin: 0 auto;
    max-width: 90em;
-webkit-box-shadow: 0 0 0 1px #AAAAAA, 0 0 5px 0 #AAAAAA;
   -moz-box-shadow: 0 0 0 1px #AAAAAA, 0 0 5px 0 #AAAAAA;
        box-shadow: 0 0 0 1px #AAAAAA, 0 0 5px 0 #AAAAAA;
}

body[dir="rtl"] > #wrapper {
-webkit-box-shadow: 0 1px 0 0 #AAAAAA, 0 0 5px 0 #AAAAAA;
   -moz-box-shadow: 0 1px 0 0 #AAAAAA, 0 0 5px 0 #AAAAAA;
        box-shadow: 0 1px 0 0 #AAAAAA, 0 0 5px 0 #AAAAAA;
}

.langauge_banner {
    color: #FFFFFF;
    height: 16em;
    margin: 0;
    vertical-align: middle;
    width: 100%;
}
.popular_books {
    color: #FFFFFF;
    height: 6em;
    text-align: center;
}
.bottom_products .title span {
}
.bottom_products > .title a {
    color: #999999 !important;
    font-size: 30px;
    text-decoration: none;
}
.bottom_products > .title a:hover {
    color: #333333 !important;
}

.bottom_versions > .title a {
    color: #666666 !important;
    font-size: 2em;
    padding-left: 0.5em;
    text-decoration: none;
}

body[dir="rtl"] .bottom_versions > .title a {
    padding-right: 0.5em;
}

#book_lang_menu,
#menu,
.breadcrumb,
#book_format_menu {
    height: 32px !important;
}
#menu {
    z-index: 999;
/*box-shadow*/
-webkit-box-shadow: 0 -1px 8px #888888, 0 -30px 30px -20px #CCCCCC inset;
   -moz-box-shadow: 0 -1px 8px #888888, 0 -30px 30px -20px #CCCCCC inset;
        box-shadow: 0 -1px 8px #888888, 0 -30px 30px -20px #CCCCCC inset;
    background-color:#EEEEEE;
}

body[dir="rtl"] #menu {
-webkit-box-shadow: 0 -1px 8px #888888, 0 -20px 30px -30px #CCCCCC inset;
   -moz-box-shadow: 0 -1px 8px #888888, 0 -20px 30px -30px #CCCCCC inset;
        box-shadow: 0 -1px 8px #888888, 0 -20px 30px -30px #CCCCCC inset;
}

.breadcrumb {
    background-color: #EEEEEE;
    color: #444444;
    cursor: pointer !important;
    display: block;
    float: left;
    line-height: 32px;
    position: relative;
    text-indent: 32px;
}

body[dir="rtl"] .breadcrumb {
    float: right;
}

.breadcrumb:before {
    border-color: transparent transparent transparent rgba(0, 0, 0, 0.2);
    border-style: solid;
    border-width: 16px 0 16px 16px;
    content: "";
    display: block;
    height: 0;
    position: absolute;
    right: -16px;
    top: 0;
    width: 0;
    z-index: 99;
}

body[dir="rtl"] .breadcrumb:before {
    border-color: transparent rgba(0, 0, 0, 0.2) transparent transparent;
    border-width: 16px 16px 16px 0;
    right: auto;
    left: -16px;
}

.breadcrumb:after {
    border-color: transparent transparent transparent #EEEEEE;
    border-style: solid;
    border-width: 16px 0 16px 16px;
    content: "";
    display: block;
    height: 0;
    position: absolute;
    right: -15px;
    top: 0;
    width: 0;
    z-index: 99;
}

body[dir="rtl"] .breadcrumb:after {
    border-color: transparent #EEEEEE transparent transparent;
    border-width: 16px 16px 16px 0;
    right: auto;
    left: -15px;
}

.breadcrumb:hover {
    background-color: #666666 !important;
    color: #FFFFFF;
}
.breadcrumb:hover:after {
    border-color: transparent transparent transparent #666666;
}

body[dir="rtl"] .breadcrumb:hover:after {
    border-color: transparent #666666 transparent transparent;
}


#book_menu, #product_menu, #version_menu {
}
#book_lang_menu,
#book_format_menu {
    background-color: #EEEEEE;
}

#book_lang_menu > div,
#book_format_menu > div {
    color: #444444;
    text-indent: 8px;
}

#book_lang_menu:hover,
#book_lang_menu:hover > div,
#book_format_menu:hover > div,
#book_format_menu:hover {
    color: white;
    background-color: #666666;
}


#search_box {
    background-color: transparent;
    padding: 0;
    width: 190px;
    position: absolute;
    right: 40px;
    z-index: 9;
}

body[dir="rtl"] #search_box {
    right: auto;
    left: 40px;
}


#search_box input[type="text"] {
    background-color: #FFFFFF;
    border: 1px solid #CCCCCC;
    display: block;
    float: left;
    height: 26px;
    margin: 2px 3px !important;
    width: 190px;
    padding: 0;
}

#search_box input[type="text"] {
    float: right;
}

#search_box input[type="submit"] {
    display: none;
}
#search_box:hover input[type="text"] {
    background-color: #FFFFFF;
    border-color: #444444;
}
body > #wrapper {
    max-width: 70em !important;
}
body > #wrapper h1 {
    font-weight: normal;
    padding-top: 20px;
    text-align: center;
}

body > #wrapper h2 {
    font-weight: normal;
    text-align: center;
}

body > #wrapper .book_top_box {
    margin: 2em auto;
    max-width: 56em;
    text-align: center;
}
.book_formats {
    background-color: transparent;
    margin: 0;
    padding: 0 1em 0 0;
    position: relative !important;
    text-align: center;
    width: 100% !important;
}

body[dir="rtl"] .book_formats {
    padding: 0 0 0 1em;
}

.book_formats div {
    display: inline;
    position: relative !important;
}
.book_formats div a {
    background-color: #777777;
    display: block;
    float: right;
    height: 128px;
    line-height: 179px;
    margin: 0.52em;
    position: relative !important;
    text-align: center;
    text-decoration: none;
    width: 12.5555%;
}

body[dir="rtl"] .book_formats div a {
    float: left;
}

.book_versions {
    border-bottom: 1px solid #CCCCCC;
    float: none;
    height: 48px;
    margin-bottom: 1em;
    position: relative;
    width: 100%;
}
.book_formats:after, .book_versions {
    clear: both;
    content: "";
    display: block;
    width: 100%;
}
.book_version:before {
    content: "Version: ";
}
.book_version {
    background-color: #EEEEEE;
    border: medium none;
    height: 48px;
    line-height: 48px;
    margin: 0 !important;
    padding: 0;
}
.breadcrumb ul {
    background-color: rgba(110, 110, 110, 0.95) !important;
    left: 0;
    margin: 22px 0 !important;
/*    padding: 0 !important; */
    top: 0;
    text-indent: 16px;
}

body[dir="rtl"] .breadcrumb ul {
    left: auto;
    right: 0;
}

.breadcrumb ul li {
    margin: 0;
    padding: 0;
}
.breadcrumb ul a {
    display: block;
    height: 32px;
    line-height: 32px;
    text-decoration: none;
    padding-right: 16px;
}

body[dir="rtl"] .breadcrumb ul a {
    padding-right: 0px;
    padding-left: 16px;
}

.breadcrumb ul a:hover {
    background-color: #333333;
}
.bottom_versions {
    margin: 0 !important;
    padding: 0 !important;
}
.bottom_books {
    display: block;
    float: left;
    font-size: 12px;
    margin-bottom: 0.5em;
    margin-left: 20px;
    margin-right: 10px;
    max-width: 50%;
    min-width: 50%;
    display: inline;
    margin: 0;
    padding: 0;
-webkit-box-sizing: border-box !important;
   -moz-box-sizing: border-box !important;
        box-sizing: border-box !important;
    height: 68px;
    padding-right:2em;
    overflow: hidden;
}

body[dir="rtl"] .bottom_books {
    float: right;
    margin-right: 0px;
    margin-left: 0px;
    padding-right: 0px;
    padding-left: 2em;
}

.bottom_books a {
    color: #333333;
    display: block;
    font-size: 12px;
    font-weight: bold;
    line-height: 12px;
    margin: 16px 0 0 0 !important;
    position: relative;
    text-decoration: none !important;
    z-index: 6;
    padding-right: 1px;
}

body[dir="rtl"] .bottom_books a {
    padding-left: 1px;
}

.bottom_books .subtitle {
    display:block;
    font-size: 12px;
    font-weight: normal;
    margin-top: 5px;
    line-height: 12px;
    color: #666;
}

.bottom_versions div:nth-child(2n) a {
}
.langauge_banner, .popular_books {
    border: medium none !important;
}
.popular_books {
    background-color: #DDDDDD;
    height: 128px;
    line-height: 128px;
    margin-bottom: 48px;
    text-shadow: 0 0 2px rgba(0, 0, 0, 0.2);
}
div.version_tabs {
}
.version_tabs span:before {
    border-width: 11px !important;
}
.version_tabs {
    margin: 18px 0;
}
.version_tabs span {
    background-color: transparent;
}
.version_tabs span a {
    min-width: 3em;
    text-align: center;
    text-decoration: none;
}
.version_tabs span a:hover {
    color: #000000;
}
.langauge_banner {
    color: rgba(0, 0, 0, 0.4);
    height: 240px;
    text-shadow: 0 1px 1px #000000;
}
.side_products div.link,
.side_products div.link a {
    color: #999999;
    cursor: pointer;
    display: block !important;
/*     height: 32px !important;
    line-height: 32px !important; */
}

.side_products div.link a {
    padding-top: 6px;
    padding-bottom: 6px;
}
.side_products div.link a:after {
    border-width: 16px !important;
    right: -31px !important;
}

body[dir="rtl"] .side_products div.link a:after {
    right: auto !important;
    left: -12px !important;
}

#slider {
    display: block;
    height: 100%;
    position: relative;
}
.slider_item {
    background-color: #990000;
    height: 100%;
    vertical-align: middle;
}
.slider_link {
    display: block;
    height: 100%;
    position: relative;
    text-decoration: none;
}
.slider_title {
    color: white;
    display: block;
    font-size: 30px;
    position: relative;
}
.slider_body {
    color: white;
    font-size: 16px;
    line-height: 22px;
    margin-left: 5%;
    padding-top: 10px;
    position: relative;
    width: 95%;
}
.slider_wrapper {
    height: 100%;
    left: 48%;
    margin: 10px;
    position: absolute;
    text-align: center;
    top: 50%;
    width: 660px;
}
.slider_wrapper2 {
    left: -50%;
    position: relative;
    top: -25%;
}
.jcarousel-container {
}
.jcarousel-container:hover .jcarousel-prev-horizontal {
    opacity: 0.4;
}
.jcarousel-container:hover .jcarousel-next-horizontal {
    opacity: 0.4;
}
.jcarousel-direction-rtl {
    direction: rtl;
}
.jcarousel-container-horizontal {
    height: 100%;
    width: 100%;
}
.jcarousel-container-vertical {
    height: 245px;
    padding: 40px 20px;
    width: 75px;
}
.jcarousel-clip {
    overflow: hidden;
}
.jcarousel-clip-horizontal {
    height: 100%;
    width: 100%;
}
.jcarousel-clip-vertical {
    height: 245px;
    width: 75px;
}
.jcarousel-item {
    width: 75px;
}
.jcarousel-item-horizontal {
    margin-left: 0;
}
.jcarousel-direction-rtl .jcarousel-item-horizontal {
/*    margin-left: 10px; */
    margin-right: 0;
}
.jcarousel-item-vertical {
    margin-bottom: 10px;
}
.jcarousel-item-placeholder {
    background: none repeat scroll 0 0 #FFFFFF;
    color: #000000;
}
.jcarousel-next-horizontal {
    background-color: #DDDDDD;
    cursor: pointer;
    height: 100%;
    opacity: 0;
    position: absolute;
    right: -32px;
    top: 0;
    width: 32px;
}
.jcarousel-next-horizontal:after {
    border-color: transparent transparent transparent #999999;
    border-style: inset;
    border-width: 10px;
    content: " ";
    height: 0;
    left: 50%;
    margin-left: -3px;
    margin-top: -10px;
    position: absolute;
    top: 50%;
    width: 0;
}
.jcarousel-direction-rtl .jcarousel-next-horizontal {
    left: -32px;
    right: auto;
}
.jcarousel-next-horizontal:hover, .jcarousel-next-horizontal:focus {
    opacity: 0.8 !important;
}
.jcarousel-next-horizontal:active {
    background-position: -64px 0;
}
.jcarousel-next-disabled-horizontal, .jcarousel-next-disabled-horizontal:hover, .jcarousel-next-disabled-horizontal:focus, .jcarousel-next-disabled-horizontal:active {
    background-color: pink;
    background-position: -96px 0;
    cursor: default;
}
.jcarousel-prev-horizontal {
    background-color: #DDDDDD;
    cursor: pointer;
    height: 100%;
    left: -32px;
    opacity: 0;
    position: absolute;
    top: 0;
    width: 32px;
}
.jcarousel-prev-horizontal:after {
    border-color: transparent black transparent transparent;
    border-style: inset;
    border-width: 10px;
    content: " ";
    height: 0;
    left: 50%;
    margin-left: -15px;
    margin-top: -10px;
    position: absolute;
    top: 50%;
    width: 0;
    opacity: 1 !important;
}
.jcarousel-direction-rtl .jcarousel-prev-horizontal {
    left: auto;
    right: -32px;
}
.jcarousel-prev-horizontal:hover, .jcarousel-prev-horizontal:focus {
    opacity: 1 !important;
}
.jcarousel-prev-horizontal:active {
    background-position: -64px 0;
}
.jcarousel-prev-disabled-horizontal, .jcarousel-prev-disabled-horizontal:hover, .jcarousel-prev-disabled-horizontal:focus, .jcarousel-prev-disabled-horizontal:active {
    background-color: pink;
    background-position: -96px 0;
    cursor: default;
}
.version_tabs .link:last-child a {
}
.version_tabs .link:first-child a {
}

.bottom_versions {
    background-color: white;
    float: left;
    padding: 40px !important;
    padding-top: 10px !important;
    min-width: 80%;
    line-height: 1.3em;
    padding-left: 1em;
    position: relative;
}

body[dir="rtl"] .bottom_versions {
    padding: 40px !important;
    padding-top: 10px !important;
    float: right;
}

.product_bottom_box .bottom_versions {
    display: block;
    min-width: 91.5%;
 }

.footer {
    display: block;
    float: left;
    width: 100%;
}

body[dir="rtl"] .footer {
    float: right;
}

.menu_embeded #title.hidden {
    display: block;
}

p, div.para, div.formalpara {
    margin-bottom: 1em;
    margin-top: 0.3em;
    padding-bottom: 0;
    padding-top: 0;
}



/*    *****    */
body,html {
	height: 100%;
}

/*needed for the hover stuff*/
/*selection fixing -now they press ctrl+a and only the document contents are selectable*/
/* breaks selection
body, section {
	-webkit-user-select: none;
	-moz-user-select: none;
	-ms-user-select: none;
	user-select: none;
}
*/

body > div.book, #main > div.book,body > div.article, #main > div.article,body > div.set, #main > div.set,body > div.preface, #main > div.preface,body > div.chapter, #main > div.chapter,body > div.section, #main > div.section,body > div.appendix, #main > div.appendix,body > div.part {
	margin-top: 0px;
/* breaks selection
	-webkit-user-select: text;
	-moz-user-select: text;
	-ms-user-select: text;
	user-select: text;
*/
	-webkit-box-shadow: 0 .5em 5em #ccc;
	box-shadow: 0 .5em 5em #ccc;
	padding: 6em;
	padding-top: 2em;
}

/*
putting the doc nav away from the site nav
 */
/*give the docnav no gradient background and a nicer shadow */
#menu {
	-webkit-box-shadow: none;
	box-shadow: none;
	border-bottom: 1px solid #ccc; /*and a nice border bottom*/
position: fixed;

	-webkit-transition: all .25s ease;
	-moz-transition: all .25s ease;
	-o-transition: all .25s ease;
	-ms-transition: all .25s ease;
	transition: all .25s ease;

	-webkit-transition-delay: 2s;
	-moz-transition-delay: 2s;
	-o-transition-delay: 2s;
	-ms-transition-delay: 2s;
	transition-delay: 2s;
	top: -32px;
}

/*setting the docnav at the bottom with a dark background */
.docnav {
	bottom: 0px;
	margin-bottom: 0em;
	background-color: rgba(1,1,1,0.75);
	color: #fff;
	width: 90%;
	left: 5%;
	height: 3em;

	-webkit-box-sizing: border-box;
	-moz-box-sizing: border-box;
	-ms-box-sizing: border-box;
	box-sizing: border-box;
	border-radius: .5em .5em 0 0;
	padding-top: .26em;
}

body.toc_embeded .docnav {
	left: auto;
	position: fixed;
	margin-left: -4em;
	width: 65em;
}

/*fancy docnav animation*/
.docnav {
	z-index: 99;
	bottom: -3em;
	margin-bottom: 0 !important;
	position: fixed;

	-webkit-transition: all .25s ease;
	-moz-transition: all .25s ease;
	-o-transition: all .25s ease;
	-ms-transition: all .25s ease;
	transition: all .25s ease;

	-webkit-transition-delay: 2s;
	-moz-transition-delay: 2s;
	-o-transition-delay: 2s;
	-ms-transition-delay: 2s;
	transition-delay: 2s;
}

body:hover .docnav {
	bottom: 0px;

	-webkit-transition-delay: 0s !important;
	-moz-transition-delay: 0s !important;
	-o-transition-delay: 0s !important;
	-ms-transition-delay: 0s !important;
	transition-delay: 0s !important;

	-webkit-transition: bottom .25s ease;
	-moz-transition: bottom .25s ease;
	-o-transition: bottom .25s ease;
	-ms-transition: bottom .25s ease;
	transition: bottom .25s ease;
}

body:hover #menu {
	top: 0px;

	-webkit-transition: bottom .25s ease;
	-moz-transition: bottom .25s ease;
	-o-transition: bottom .25s ease;
	-ms-transition: bottom .25s ease;
	transition: bottom .25s ease;

	-webkit-transition-delay: 0s !important;
	-moz-transition-delay: 0s !important;
	-o-transition-delay: 0s !important;
	-ms-transition-delay: 0s !important;
	transition-delay: 0s !important;
}



/*making the text white */
.docnav li.next a strong,.docnav li.previous a strong,.docnav .home {
	color: #999;
}

.docnav li.next a strong:hover,.docnav li.previous a strong:hover,.docnav .home:hover {
	color: #fff;
}

/*making the docnav  next prev buttons spacier */
.docnav li.next a strong,.docnav li.previous a strong {
	padding: .3em 4em;
	line-height: 2em;
}

.docnav li.next a strong {
	background-position: 90% 90%;
}

.docnav li.previous a strong {
	background-position: 10% 90%;
}

#book_lang_menu,#lang_menu_2,#book_format_menu {
	border: none !important;
	background-color: transparent;
}

/*fixing the docnav arrow*/
.home {
	padding: .5em !important;
}

.home::after {
	top: 0.8em;
	border-width: 0em .5em .5em .5em;
	border-color: transparent transparent #999 transparent;
}

/*tightening up the breadcrumb*/
.breadcrumb {
	text-indent: 1em;
}

/*tightening up the language and formats positioning*/
#book_lang_menu,#lang_menu_2,#book_format_menu {
	right: 140px;
	z-index: 0;
}

/*awesome animated text box*/
#search_box,#search_box input[type="text"] {
	width: 90px;

	-webkit-transition: all .25s ease;
	-moz-transition: all .25s ease;
	-o-transition: all .25s ease;
	-ms-transition: all .25s ease;
	transition: all .25s ease;
	text-indent: .5em;
}

#search_box input[type="text"]:focus,#search_box input[type="text"]:hover {
	width: 320px;
}

/*fixing up the floating toc*/
/*animation*/
#floatingtoc a {
	color: #fff;
}

#floatingtoc dd a {
	color: #ccc;
}

#floatingtoc {
	border: none;
	padding: 5em 0em;
	bottom: 0px;

	-webkit-transition-delay: 2s;
	-moz-transition-delay: 2s;
	-o-transition-delay: 2s;
	-ms-transition-delay: 2s;
	transition-delay: 2s;

	-webkit-transition: all .2s ease;
	-moz-transition: all .2s ease;
	-o-transition: all .2s ease;
	-ms-transition: all .2s ease;
	transition: all .2s ease;
	background-color: #fff;
	background-color: rgba(255,255,255,0.86);
	max-height: none;
	top: 0px;
	width: 100%;
	margin: 0em;
	overflow: hidden;
	left: 0em;
}

#floatingtoc .toc {
	overflow: auto;
	width: 70% !important;
 	padding: 2em;

	-webkit-box-sizing: border-box;
	-moz-box-sizing: border-box;
	-ms-box-sizing: border-box;
	box-sizing: border-box;
	margin: 0 auto;
	background-color: #333;
	background-color: rgba(10,10,10,0.83);
	border-radius: .51em .51em 0em 0em;
	position: fixed;
	top: 50px;
	left: 50%;
	margin-left: -30% !important;
	bottom: 0px;

	-webkit-transition: bottom .2s ease;
	-moz-transition: bottom .2s ease;
	-o-transition: bottom .2s ease;
	-ms-transition: bottom .2s ease;
	transition: bottom .2s ease;
	-webkit-transition-delay: 2s;
	-moz-transition-delay: 2s;
	-o-transition-delay: 2s;
	-ms-transition-delay: 2s;
	transition-delay: 2s;
}

body.toc_embeded #floatingtoc .toc {
	width: 60% !important;
	left: 50%;
	margin-left:  -20%
}

body:hover #floatingtoc .toc {
	bottom: 3em;
	-webkit-transition: bottom .2s ease;
	-moz-transition: bottom .2s ease;
	-o-transition: bottom .2s ease;
	-ms-transition: bottom .2s ease;
	transition: bottom .2s ease;
	-webkit-transition-delay: 2s;
	-moz-transition-delay: 2s;
	-o-transition-delay: 2s;
	-ms-transition-delay: 2s;
	transition-delay: 2s;
}

/*

/*   #floatingtoc .toc:after {
/*   	-webkit-transition: all .2s ease;
/*   	-moz-transition: all .2s ease;
/*   	-o-transition: all .2s ease;
/*   	-ms-transition: all .2s ease;
/*   	transition: all .2s ease;
/*   	content: "";
/*   	display: block;
/*   	width: 0;
/*   	height: 0;
/*   	border-width: .5em;
/*   	border-color: transparent transparent #fff transparent;
/*   	border-style: solid;
/*   	z-index: 99;
/*   	position: fixed;
/*   	bottom: 0px;
/*   	left: 50%;
/*   	margin-left: -.25em;
/*   }
*/

body:hover #floatingtoc .toc:after {
	bottom: 39px;

	-webkit-transition: all .2s ease;
	-moz-transition: all .2s ease;
	-o-transition: all .2s ease;
	-ms-transition: all .2s ease;
	transition: all .2s ease;
}

#menu ul li {
	margin-left: 0;
	margin-right: 0;
	text-indent: 25px;
}

#menu ul a {
	color: #ddd;
}

#menu ul a:hover {
	background-color: #666;
	color: #FFF !important;
}


/*this media query hack makes sure tablets and phones always get the menus */
@media only screen and (max-width: 65em) {
	body .docnav {
	bottom: 0px !important;
}



body #menu {
	top: 0px !important;}}

@media only screen and (max-width: 45em) {

	body > div.book, #main > div.book, body > div.article, #main > div.article, body > div.set, #main > div.set, body > div.preface, #main > div.preface, body > div.chapter, #main > div.chapter, body > div.section, #main > div.section, body > div.appendix, #main > div.appendix, body > div.part {padding:2em;}
#book_lang_menu, #lang_menu_2, #book_format_menu{right: 0 !important;}

}

#book_lang_menu:hover, #book_lang_menu:hover > div, #book_format_menu:hover > div, #book_format_menu:hover{cursor:pointer} /*makes sure the lang and formats menu has the right cursor*/

p#title {
	padding: 1em;
	margin: 0 auto;
	margin-top: 110px;
	background-color: transparent;
	border: none !important;
}

body.toc_embeded p#title {
	margin-top: 55px;
}

.draft .book {
	background-image: url(common/en-US/images/watermark-draft.png);
	background-repeat: repeat-y;
        background-position: center;
}

.visible {
	display: block !important;
}

.hidden {
	display: none !important;
}

#floatingtoc .toc {
	max-width: 40em !important;
	left: 50% !important;
	margin-left: -10em !important;
	position: absolute !important;
}


/*narrower lists in nav for smaller screens */
@media (max-height: 640px) {
    #product_menu a{font-size:.85em !important; line-height:2em; height:2em;}
    
}


/*smaller nav for smaller screens */
@media (max-width: 900px) {
    #menu {font-size:.75em !important;}}



/*fixing the overflow of long product menus*/
#product_menu ul,
#book_menu ul,
#version_menu ul,
#lang_menu_2 ul {
	left: inherit;
	position:fixed !important;
	max-height: 90%;
	display:block;
	top:10px !important;
	margin-left: -.9em !important;
	max-width: 70em !important;
	background-color: pink;
	overflow: auto;
	min-width: 200px !important
}

#lang_menu_2 ul {
top: 32px !important;
}


/* shrink the font size of the docunav when viewing on a small screen*/
@media (max-width: 900px) {
     .docnav , .docnav li.home,  .docnav a strong, #floatingtoc{font-size:.75em !important;}
    }
   
}



/*when there is no side nav */
#floatingtoc .toc { 
        width: 40em !important;
  left: 35% !important;
  margin-left: -20em !important;
  position: absolute !important;
}




/*when there is a side nav, nudge the floating toc to the left by 20em */
.toc_embeded #floatingtoc .toc { 
        width: 40em !important;
  left: 50% !important;
  margin-left: 0em !important;
  position: absolute !important;
}

#lang_menu_list{ right:2em;}

