@import url("../../en-US/css/menu.css");
/* RTL problems fixed by <alsadi@ojuba.org> report RTL bugs to core@ojuba.org */
pre, tt, code, div.legalnotice .para {
	text-align: left;
	direction: ltr;
}

body {
	direction: rtl;
}

body.toc_embeded {
/*
	margin-left: inherit;
	margin-right: 300px;
	text-align: right;
	float: right;
*/
}

object.toc, iframe.toc {
	left: inherit;
	right: 0;
	border-style:none;
	border-left:1px solid #999;
}

body.desktop {
	margin-left: inherit;
	margin-right: 26em;
	text-align: right;
	float: right;
}

body.desktop .book > .toc {
	padding-right:1em;
}


body.desktop .book > .toc {
	right:0px;
	padding-right:1em;
}

div.longdesc-link {
	float:left;
	color:#999;
}

.titlepage h1.title {
	text-align:right;
}

.producttitle {
}
.section h1.title {
	text-align: right;
}

.article ul {
	padding-right:2em;
}

.article li {
	padding-right:0em;
}

.books li {
	float:right;
}

.books .pdf {
	right:170px;
}

.products li {
	float:right;
}

.revhistory td {
	text-align:left;
}

.revhistory tr td:first-child {
	text-align:right;
}

.revhistory tr td p, .revhistory tr td div.para {
	text-align:right;
}

.copyleft {
	margin-top: 1em;
}

.answer .data, .question .data {
	padding-right: 2.6em;
}

.answer label, .question label {
	float:right;
	font-weight:bold;
}


ul {
	padding-right:1.6em;
}

dd {
	margin-right:2em;
}


div.note, div.important, div.warning {
	background-position: right 1em;
}

div.note pre, div.important pre, div.warning pre {
	margin-right: 4.5em;
}

div.note li, div.warning li, div.important li {
	padding-right:10px;
}

div.note ul, div.warning ul, div.important ul {
	padding-right:40px;
}

div.note h2, div.note p, div.note div.para, div.warning h2, div.warning p, div.warning div.para, div.important h2, .important p, .important div.para {
	padding-right:56px;
}

#title a.right{
	display:inline;
	border:none;
	padding-right:200px;
}

#title a.right img{
	border:none;
	float:right;
	margin:0em;
	margin-top:0em;
}

#title a.left {
	padding-bottom:1em;
}

#title a.left img {
	border:none;
	float:left;
	margin:0em;
}

table th {
	text-align:right;
}

#breadcrumbs ul {
	margin-right:0;
	padding-right:0;
}

#breadcrumbs ul li {
	margin-right:0;
	padding-right:2px;
}

.alpha1 {
	background: white url(../images/watermark-alpha1.png) top right repeat;
}

.alpha2 {
	background: white url(../images/watermark-alpha2.png) top right repeat;
}

.beta1 {
	background: white url(../images/watermark-beta1.png) top right repeat;
}

.beta2 {
	background: white url(../images/watermark-beta2.png) top right repeat;
}

.pre-release-candidate {
	background: white url(../images/watermark-pre-release-candidate.png) top right repeat;
}

.release-candidate {
	background: white url(../images/watermark-release-candidate.png) top right repeat;
}

sup {
	padding-right:0em;
}

.footnote sup  {
	right: .4em;
}

.footnote p,.footnote div.para {
	padding-right:5em;
}

dt .inlinemediaobject, dt object {
	float: right;
}


.docnav li.previous {
	right: 0px;
	left: 50em;
	text-align: right;
}

.docnav li.next {
	left: 0em;
	right: 50em;
	text-align: left;
}

.docnav li.next a strong {
	background:  url(../images/stock-go-back.png) left 120% no-repeat;
	padding-left:28px;
}

.docnav li.previous a strong {
	background: url(../images/stock-go-forward.png) right 120% no-repeat;
	padding-right:28px;
	padding-left:0.5em;
}

.docnav li.home a strong {
	background: url(../images/stock-home.png) top right no-repeat;
	padding-right:28px;
}

.docnav li.up a strong {
	background: url(../images/stock-go-up.png) top right no-repeat;
	padding-right:28px;
}

.toc dd {
	padding-right: 1.3em;
	margin-right: 0em;
}

div.toc > dl > dt {
	padding-bottom: 0em;
	margin-bottom: 0em;
	margin-top: 1em;
}

